/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
* @brief Example template project.
* @defgroup nrf_templates_example Example Template
*
*/

#include <stdbool.h>
#include "nrf_gpio.h"
#include "nrf_drv_twi.h"
#include "bsp.h"
#include "app_error.h"
#include "nrf_drv_config.h"
#include "app_util_platform.h"
#include "nrf_delay.h"

// TWI event handler. Processing TWI events originating from
// nrf_drv_twi_tx()
static void twi_event_handler(nrf_drv_twi_evt_t *evt){
     
    // Light up LED 1 if device found
    if(evt->type == NRF_DRV_TWI_TX_DONE)
    {
        nrf_gpio_pin_clear(LED_1);
    }
}

static void gpio_init()
{
    nrf_gpio_cfg_output(LED_1);
    nrf_gpio_pin_set(LED_1);
}


/**
 * @brief Function for application main entry.
 */
int main(void)
{
    gpio_init();
    
    ret_code_t ret_code; 

    const nrf_drv_twi_t           p_twi_instance = NRF_DRV_TWI_INSTANCE(1); // Set up TWI instance 1 with default values
    
    // EITHER CONFIGURE THE TWI MODULE WITH DEFAULT VALUES:
    nrf_drv_twi_config_t    p_twi_config = NRF_DRV_TWI_DEFAULT_CONFIG(1);// Set up TWI configuration default values
    
    // OR CHOOSE YOU OWN TWI MODULE SETTINGS LIKE THIS:
//    nrf_drv_twi_config_t    p_twi_config;
//    p_twi_config.scl = 1;
//    p_twi_config.sda = 2;
//    p_twi_config.frequency = NRF_TWI_FREQ_400K;
//    p_twi_config.interrupt_priority = APP_IRQ_PRIORITY_LOW;

    ret_code = nrf_drv_twi_init(&p_twi_instance, &p_twi_config, twi_event_handler); // Initiate twi driver with instance and configuration values
    APP_ERROR_CHECK(ret_code); // Check for errors in return value
    
    nrf_drv_twi_enable(&p_twi_instance); // Enable the TWI instance

    const uint8_t dummy_data = 0x55; // Declare some dummy data to use for our search for devices on the TWI bus
    
    // Make a loop iterating through all TWI address listening to a response. x
    // The response will be reported in twi_event_handler()  
    for(uint8_t i=0; i < 0x7f; i++) 
    {
        // Send dummy data on TWI buss on address 'i'
        ret_code = nrf_drv_twi_tx(&p_twi_instance, i, &dummy_data, 1, false);

        // Small delay or the driver might not be able to keep up with the fast transmissions
        // There is deffinitely a better way to do this
        nrf_delay_ms(5); 
    }
    while (true)
    {
        // Do nothing
    }
}
/** @} */
